"""

THIS CODE IS AUTO-GENERATED. DO NOT EDIT.

Constants for OpenGL ES 2.0.

"""


class Enum(int):
    ''' Enum (integer) with a meaningfull repr. '''
    def __new__(cls, name, value):
        base = int.__new__(cls, value)
        base.name = name
        return base
    def __repr__(self):
        return self.name


GL_ACTIVE_ATTRIBUTES = Enum('GL_ACTIVE_ATTRIBUTES', 35721)
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = Enum('GL_ACTIVE_ATTRIBUTE_MAX_LENGTH', 35722)
GL_ACTIVE_TEXTURE = Enum('GL_ACTIVE_TEXTURE', 34016)
GL_ACTIVE_UNIFORMS = Enum('GL_ACTIVE_UNIFORMS', 35718)
GL_ACTIVE_UNIFORM_MAX_LENGTH = Enum('GL_ACTIVE_UNIFORM_MAX_LENGTH', 35719)
GL_ALIASED_LINE_WIDTH_RANGE = Enum('GL_ALIASED_LINE_WIDTH_RANGE', 33902)
GL_ALIASED_POINT_SIZE_RANGE = Enum('GL_ALIASED_POINT_SIZE_RANGE', 33901)
GL_ALPHA = Enum('GL_ALPHA', 6406)
GL_ALPHA_BITS = Enum('GL_ALPHA_BITS', 3413)
GL_ALWAYS = Enum('GL_ALWAYS', 519)
GL_ARRAY_BUFFER = Enum('GL_ARRAY_BUFFER', 34962)
GL_ARRAY_BUFFER_BINDING = Enum('GL_ARRAY_BUFFER_BINDING', 34964)
GL_ATTACHED_SHADERS = Enum('GL_ATTACHED_SHADERS', 35717)
GL_BACK = Enum('GL_BACK', 1029)
GL_BLEND = Enum('GL_BLEND', 3042)
GL_BLEND_COLOR = Enum('GL_BLEND_COLOR', 32773)
GL_BLEND_DST_ALPHA = Enum('GL_BLEND_DST_ALPHA', 32970)
GL_BLEND_DST_RGB = Enum('GL_BLEND_DST_RGB', 32968)
GL_BLEND_EQUATION = Enum('GL_BLEND_EQUATION', 32777)
GL_BLEND_EQUATION_ALPHA = Enum('GL_BLEND_EQUATION_ALPHA', 34877)
GL_BLEND_EQUATION_RGB = Enum('GL_BLEND_EQUATION_RGB', 32777)
GL_BLEND_SRC_ALPHA = Enum('GL_BLEND_SRC_ALPHA', 32971)
GL_BLEND_SRC_RGB = Enum('GL_BLEND_SRC_RGB', 32969)
GL_BLUE_BITS = Enum('GL_BLUE_BITS', 3412)
GL_BOOL = Enum('GL_BOOL', 35670)
GL_BOOL_VEC2 = Enum('GL_BOOL_VEC2', 35671)
GL_BOOL_VEC3 = Enum('GL_BOOL_VEC3', 35672)
GL_BOOL_VEC4 = Enum('GL_BOOL_VEC4', 35673)
GL_BUFFER_SIZE = Enum('GL_BUFFER_SIZE', 34660)
GL_BUFFER_USAGE = Enum('GL_BUFFER_USAGE', 34661)
GL_BYTE = Enum('GL_BYTE', 5120)
GL_CCW = Enum('GL_CCW', 2305)
GL_CLAMP_TO_EDGE = Enum('GL_CLAMP_TO_EDGE', 33071)
GL_COLOR_ATTACHMENT0 = Enum('GL_COLOR_ATTACHMENT0', 36064)
GL_COLOR_BUFFER_BIT = Enum('GL_COLOR_BUFFER_BIT', 16384)
GL_COLOR_CLEAR_VALUE = Enum('GL_COLOR_CLEAR_VALUE', 3106)
GL_COLOR_WRITEMASK = Enum('GL_COLOR_WRITEMASK', 3107)
GL_COMPILE_STATUS = Enum('GL_COMPILE_STATUS', 35713)
GL_COMPRESSED_TEXTURE_FORMATS = Enum('GL_COMPRESSED_TEXTURE_FORMATS', 34467)
GL_CONSTANT_ALPHA = Enum('GL_CONSTANT_ALPHA', 32771)
GL_CONSTANT_COLOR = Enum('GL_CONSTANT_COLOR', 32769)
GL_CULL_FACE = Enum('GL_CULL_FACE', 2884)
GL_CULL_FACE_MODE = Enum('GL_CULL_FACE_MODE', 2885)
GL_CURRENT_PROGRAM = Enum('GL_CURRENT_PROGRAM', 35725)
GL_CURRENT_VERTEX_ATTRIB = Enum('GL_CURRENT_VERTEX_ATTRIB', 34342)
GL_CW = Enum('GL_CW', 2304)
GL_DECR = Enum('GL_DECR', 7683)
GL_DECR_WRAP = Enum('GL_DECR_WRAP', 34056)
GL_DELETE_STATUS = Enum('GL_DELETE_STATUS', 35712)
GL_DEPTH_ATTACHMENT = Enum('GL_DEPTH_ATTACHMENT', 36096)
GL_DEPTH_BITS = Enum('GL_DEPTH_BITS', 3414)
GL_DEPTH_BUFFER_BIT = Enum('GL_DEPTH_BUFFER_BIT', 256)
GL_DEPTH_CLEAR_VALUE = Enum('GL_DEPTH_CLEAR_VALUE', 2931)
GL_DEPTH_COMPONENT = Enum('GL_DEPTH_COMPONENT', 6402)
GL_DEPTH_COMPONENT16 = Enum('GL_DEPTH_COMPONENT16', 33189)
GL_DEPTH_FUNC = Enum('GL_DEPTH_FUNC', 2932)
GL_DEPTH_RANGE = Enum('GL_DEPTH_RANGE', 2928)
GL_DEPTH_TEST = Enum('GL_DEPTH_TEST', 2929)
GL_DEPTH_WRITEMASK = Enum('GL_DEPTH_WRITEMASK', 2930)
GL_DITHER = Enum('GL_DITHER', 3024)
GL_DONT_CARE = Enum('GL_DONT_CARE', 4352)
GL_DST_ALPHA = Enum('GL_DST_ALPHA', 772)
GL_DST_COLOR = Enum('GL_DST_COLOR', 774)
GL_DYNAMIC_DRAW = Enum('GL_DYNAMIC_DRAW', 35048)
GL_ELEMENT_ARRAY_BUFFER = Enum('GL_ELEMENT_ARRAY_BUFFER', 34963)
GL_ELEMENT_ARRAY_BUFFER_BINDING = Enum('GL_ELEMENT_ARRAY_BUFFER_BINDING', 34965)
GL_EQUAL = Enum('GL_EQUAL', 514)
GL_ES_VERSION_2_0 = Enum('GL_ES_VERSION_2_0', 1)
GL_EXTENSIONS = Enum('GL_EXTENSIONS', 7939)
GL_FALSE = Enum('GL_FALSE', 0)
GL_FASTEST = Enum('GL_FASTEST', 4353)
GL_FIXED = Enum('GL_FIXED', 5132)
GL_FLOAT = Enum('GL_FLOAT', 5126)
GL_FLOAT_MAT2 = Enum('GL_FLOAT_MAT2', 35674)
GL_FLOAT_MAT3 = Enum('GL_FLOAT_MAT3', 35675)
GL_FLOAT_MAT4 = Enum('GL_FLOAT_MAT4', 35676)
GL_FLOAT_VEC2 = Enum('GL_FLOAT_VEC2', 35664)
GL_FLOAT_VEC3 = Enum('GL_FLOAT_VEC3', 35665)
GL_FLOAT_VEC4 = Enum('GL_FLOAT_VEC4', 35666)
GL_FRAGMENT_SHADER = Enum('GL_FRAGMENT_SHADER', 35632)
GL_FRAMEBUFFER = Enum('GL_FRAMEBUFFER', 36160)
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = Enum('GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME', 36049)
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = Enum('GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE', 36048)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = Enum('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE', 36051)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = Enum('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL', 36050)
GL_FRAMEBUFFER_BINDING = Enum('GL_FRAMEBUFFER_BINDING', 36006)
GL_FRAMEBUFFER_COMPLETE = Enum('GL_FRAMEBUFFER_COMPLETE', 36053)
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = Enum('GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT', 36054)
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS = Enum('GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS', 36057)
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = Enum('GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT', 36055)
GL_FRAMEBUFFER_UNSUPPORTED = Enum('GL_FRAMEBUFFER_UNSUPPORTED', 36061)
GL_FRONT = Enum('GL_FRONT', 1028)
GL_FRONT_AND_BACK = Enum('GL_FRONT_AND_BACK', 1032)
GL_FRONT_FACE = Enum('GL_FRONT_FACE', 2886)
GL_FUNC_ADD = Enum('GL_FUNC_ADD', 32774)
GL_FUNC_REVERSE_SUBTRACT = Enum('GL_FUNC_REVERSE_SUBTRACT', 32779)
GL_FUNC_SUBTRACT = Enum('GL_FUNC_SUBTRACT', 32778)
GL_GENERATE_MIPMAP_HINT = Enum('GL_GENERATE_MIPMAP_HINT', 33170)
GL_GEQUAL = Enum('GL_GEQUAL', 518)
GL_GREATER = Enum('GL_GREATER', 516)
GL_GREEN_BITS = Enum('GL_GREEN_BITS', 3411)
GL_HIGH_FLOAT = Enum('GL_HIGH_FLOAT', 36338)
GL_HIGH_INT = Enum('GL_HIGH_INT', 36341)
GL_IMPLEMENTATION_COLOR_READ_FORMAT = Enum('GL_IMPLEMENTATION_COLOR_READ_FORMAT', 35739)
GL_IMPLEMENTATION_COLOR_READ_TYPE = Enum('GL_IMPLEMENTATION_COLOR_READ_TYPE', 35738)
GL_INCR = Enum('GL_INCR', 7682)
GL_INCR_WRAP = Enum('GL_INCR_WRAP', 34055)
GL_INFO_LOG_LENGTH = Enum('GL_INFO_LOG_LENGTH', 35716)
GL_INT = Enum('GL_INT', 5124)
GL_INT_VEC2 = Enum('GL_INT_VEC2', 35667)
GL_INT_VEC3 = Enum('GL_INT_VEC3', 35668)
GL_INT_VEC4 = Enum('GL_INT_VEC4', 35669)
GL_INVALID_ENUM = Enum('GL_INVALID_ENUM', 1280)
GL_INVALID_FRAMEBUFFER_OPERATION = Enum('GL_INVALID_FRAMEBUFFER_OPERATION', 1286)
GL_INVALID_OPERATION = Enum('GL_INVALID_OPERATION', 1282)
GL_INVALID_VALUE = Enum('GL_INVALID_VALUE', 1281)
GL_INVERT = Enum('GL_INVERT', 5386)
GL_KEEP = Enum('GL_KEEP', 7680)
GL_LEQUAL = Enum('GL_LEQUAL', 515)
GL_LESS = Enum('GL_LESS', 513)
GL_LINEAR = Enum('GL_LINEAR', 9729)
GL_LINEAR_MIPMAP_LINEAR = Enum('GL_LINEAR_MIPMAP_LINEAR', 9987)
GL_LINEAR_MIPMAP_NEAREST = Enum('GL_LINEAR_MIPMAP_NEAREST', 9985)
GL_LINES = Enum('GL_LINES', 1)
GL_LINE_LOOP = Enum('GL_LINE_LOOP', 2)
GL_LINE_STRIP = Enum('GL_LINE_STRIP', 3)
GL_LINE_WIDTH = Enum('GL_LINE_WIDTH', 2849)
GL_LINK_STATUS = Enum('GL_LINK_STATUS', 35714)
GL_LOW_FLOAT = Enum('GL_LOW_FLOAT', 36336)
GL_LOW_INT = Enum('GL_LOW_INT', 36339)
GL_LUMINANCE = Enum('GL_LUMINANCE', 6409)
GL_LUMINANCE_ALPHA = Enum('GL_LUMINANCE_ALPHA', 6410)
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = Enum('GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS', 35661)
GL_MAX_CUBE_MAP_TEXTURE_SIZE = Enum('GL_MAX_CUBE_MAP_TEXTURE_SIZE', 34076)
GL_MAX_FRAGMENT_UNIFORM_VECTORS = Enum('GL_MAX_FRAGMENT_UNIFORM_VECTORS', 36349)
GL_MAX_RENDERBUFFER_SIZE = Enum('GL_MAX_RENDERBUFFER_SIZE', 34024)
GL_MAX_TEXTURE_IMAGE_UNITS = Enum('GL_MAX_TEXTURE_IMAGE_UNITS', 34930)
GL_MAX_TEXTURE_SIZE = Enum('GL_MAX_TEXTURE_SIZE', 3379)
GL_MAX_VARYING_VECTORS = Enum('GL_MAX_VARYING_VECTORS', 36348)
GL_MAX_VERTEX_ATTRIBS = Enum('GL_MAX_VERTEX_ATTRIBS', 34921)
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = Enum('GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS', 35660)
GL_MAX_VERTEX_UNIFORM_VECTORS = Enum('GL_MAX_VERTEX_UNIFORM_VECTORS', 36347)
GL_MAX_VIEWPORT_DIMS = Enum('GL_MAX_VIEWPORT_DIMS', 3386)
GL_MEDIUM_FLOAT = Enum('GL_MEDIUM_FLOAT', 36337)
GL_MEDIUM_INT = Enum('GL_MEDIUM_INT', 36340)
GL_MIRRORED_REPEAT = Enum('GL_MIRRORED_REPEAT', 33648)
GL_NEAREST = Enum('GL_NEAREST', 9728)
GL_NEAREST_MIPMAP_LINEAR = Enum('GL_NEAREST_MIPMAP_LINEAR', 9986)
GL_NEAREST_MIPMAP_NEAREST = Enum('GL_NEAREST_MIPMAP_NEAREST', 9984)
GL_NEVER = Enum('GL_NEVER', 512)
GL_NICEST = Enum('GL_NICEST', 4354)
GL_NONE = Enum('GL_NONE', 0)
GL_NOTEQUAL = Enum('GL_NOTEQUAL', 517)
GL_NO_ERROR = Enum('GL_NO_ERROR', 0)
GL_NUM_COMPRESSED_TEXTURE_FORMATS = Enum('GL_NUM_COMPRESSED_TEXTURE_FORMATS', 34466)
GL_NUM_SHADER_BINARY_FORMATS = Enum('GL_NUM_SHADER_BINARY_FORMATS', 36345)
GL_ONE = Enum('GL_ONE', 1)
GL_ONE_MINUS_CONSTANT_ALPHA = Enum('GL_ONE_MINUS_CONSTANT_ALPHA', 32772)
GL_ONE_MINUS_CONSTANT_COLOR = Enum('GL_ONE_MINUS_CONSTANT_COLOR', 32770)
GL_ONE_MINUS_DST_ALPHA = Enum('GL_ONE_MINUS_DST_ALPHA', 773)
GL_ONE_MINUS_DST_COLOR = Enum('GL_ONE_MINUS_DST_COLOR', 775)
GL_ONE_MINUS_SRC_ALPHA = Enum('GL_ONE_MINUS_SRC_ALPHA', 771)
GL_ONE_MINUS_SRC_COLOR = Enum('GL_ONE_MINUS_SRC_COLOR', 769)
GL_OUT_OF_MEMORY = Enum('GL_OUT_OF_MEMORY', 1285)
GL_PACK_ALIGNMENT = Enum('GL_PACK_ALIGNMENT', 3333)
GL_POINTS = Enum('GL_POINTS', 0)
GL_POLYGON_OFFSET_FACTOR = Enum('GL_POLYGON_OFFSET_FACTOR', 32824)
GL_POLYGON_OFFSET_FILL = Enum('GL_POLYGON_OFFSET_FILL', 32823)
GL_POLYGON_OFFSET_UNITS = Enum('GL_POLYGON_OFFSET_UNITS', 10752)
GL_RED_BITS = Enum('GL_RED_BITS', 3410)
GL_RENDERBUFFER = Enum('GL_RENDERBUFFER', 36161)
GL_RENDERBUFFER_ALPHA_SIZE = Enum('GL_RENDERBUFFER_ALPHA_SIZE', 36179)
GL_RENDERBUFFER_BINDING = Enum('GL_RENDERBUFFER_BINDING', 36007)
GL_RENDERBUFFER_BLUE_SIZE = Enum('GL_RENDERBUFFER_BLUE_SIZE', 36178)
GL_RENDERBUFFER_DEPTH_SIZE = Enum('GL_RENDERBUFFER_DEPTH_SIZE', 36180)
GL_RENDERBUFFER_GREEN_SIZE = Enum('GL_RENDERBUFFER_GREEN_SIZE', 36177)
GL_RENDERBUFFER_HEIGHT = Enum('GL_RENDERBUFFER_HEIGHT', 36163)
GL_RENDERBUFFER_INTERNAL_FORMAT = Enum('GL_RENDERBUFFER_INTERNAL_FORMAT', 36164)
GL_RENDERBUFFER_RED_SIZE = Enum('GL_RENDERBUFFER_RED_SIZE', 36176)
GL_RENDERBUFFER_STENCIL_SIZE = Enum('GL_RENDERBUFFER_STENCIL_SIZE', 36181)
GL_RENDERBUFFER_WIDTH = Enum('GL_RENDERBUFFER_WIDTH', 36162)
GL_RENDERER = Enum('GL_RENDERER', 7937)
GL_REPEAT = Enum('GL_REPEAT', 10497)
GL_REPLACE = Enum('GL_REPLACE', 7681)
GL_RGB = Enum('GL_RGB', 6407)
GL_RGB565 = Enum('GL_RGB565', 36194)
GL_RGB5_A1 = Enum('GL_RGB5_A1', 32855)
GL_RGBA = Enum('GL_RGBA', 6408)
GL_RGBA4 = Enum('GL_RGBA4', 32854)
GL_SAMPLER_2D = Enum('GL_SAMPLER_2D', 35678)
GL_SAMPLER_CUBE = Enum('GL_SAMPLER_CUBE', 35680)
GL_SAMPLES = Enum('GL_SAMPLES', 32937)
GL_SAMPLE_ALPHA_TO_COVERAGE = Enum('GL_SAMPLE_ALPHA_TO_COVERAGE', 32926)
GL_SAMPLE_BUFFERS = Enum('GL_SAMPLE_BUFFERS', 32936)
GL_SAMPLE_COVERAGE = Enum('GL_SAMPLE_COVERAGE', 32928)
GL_SAMPLE_COVERAGE_INVERT = Enum('GL_SAMPLE_COVERAGE_INVERT', 32939)
GL_SAMPLE_COVERAGE_VALUE = Enum('GL_SAMPLE_COVERAGE_VALUE', 32938)
GL_SCISSOR_BOX = Enum('GL_SCISSOR_BOX', 3088)
GL_SCISSOR_TEST = Enum('GL_SCISSOR_TEST', 3089)
GL_SHADER_BINARY_FORMATS = Enum('GL_SHADER_BINARY_FORMATS', 36344)
GL_SHADER_COMPILER = Enum('GL_SHADER_COMPILER', 36346)
GL_SHADER_SOURCE_LENGTH = Enum('GL_SHADER_SOURCE_LENGTH', 35720)
GL_SHADER_TYPE = Enum('GL_SHADER_TYPE', 35663)
GL_SHADING_LANGUAGE_VERSION = Enum('GL_SHADING_LANGUAGE_VERSION', 35724)
GL_SHORT = Enum('GL_SHORT', 5122)
GL_SRC_ALPHA = Enum('GL_SRC_ALPHA', 770)
GL_SRC_ALPHA_SATURATE = Enum('GL_SRC_ALPHA_SATURATE', 776)
GL_SRC_COLOR = Enum('GL_SRC_COLOR', 768)
GL_STATIC_DRAW = Enum('GL_STATIC_DRAW', 35044)
GL_STENCIL_ATTACHMENT = Enum('GL_STENCIL_ATTACHMENT', 36128)
GL_STENCIL_BACK_FAIL = Enum('GL_STENCIL_BACK_FAIL', 34817)
GL_STENCIL_BACK_FUNC = Enum('GL_STENCIL_BACK_FUNC', 34816)
GL_STENCIL_BACK_PASS_DEPTH_FAIL = Enum('GL_STENCIL_BACK_PASS_DEPTH_FAIL', 34818)
GL_STENCIL_BACK_PASS_DEPTH_PASS = Enum('GL_STENCIL_BACK_PASS_DEPTH_PASS', 34819)
GL_STENCIL_BACK_REF = Enum('GL_STENCIL_BACK_REF', 36003)
GL_STENCIL_BACK_VALUE_MASK = Enum('GL_STENCIL_BACK_VALUE_MASK', 36004)
GL_STENCIL_BACK_WRITEMASK = Enum('GL_STENCIL_BACK_WRITEMASK', 36005)
GL_STENCIL_BITS = Enum('GL_STENCIL_BITS', 3415)
GL_STENCIL_BUFFER_BIT = Enum('GL_STENCIL_BUFFER_BIT', 1024)
GL_STENCIL_CLEAR_VALUE = Enum('GL_STENCIL_CLEAR_VALUE', 2961)
GL_STENCIL_FAIL = Enum('GL_STENCIL_FAIL', 2964)
GL_STENCIL_FUNC = Enum('GL_STENCIL_FUNC', 2962)
GL_STENCIL_INDEX8 = Enum('GL_STENCIL_INDEX8', 36168)
GL_STENCIL_PASS_DEPTH_FAIL = Enum('GL_STENCIL_PASS_DEPTH_FAIL', 2965)
GL_STENCIL_PASS_DEPTH_PASS = Enum('GL_STENCIL_PASS_DEPTH_PASS', 2966)
GL_STENCIL_REF = Enum('GL_STENCIL_REF', 2967)
GL_STENCIL_TEST = Enum('GL_STENCIL_TEST', 2960)
GL_STENCIL_VALUE_MASK = Enum('GL_STENCIL_VALUE_MASK', 2963)
GL_STENCIL_WRITEMASK = Enum('GL_STENCIL_WRITEMASK', 2968)
GL_STREAM_DRAW = Enum('GL_STREAM_DRAW', 35040)
GL_SUBPIXEL_BITS = Enum('GL_SUBPIXEL_BITS', 3408)
GL_TEXTURE = Enum('GL_TEXTURE', 5890)
GL_TEXTURE0 = Enum('GL_TEXTURE0', 33984)
GL_TEXTURE1 = Enum('GL_TEXTURE1', 33985)
GL_TEXTURE10 = Enum('GL_TEXTURE10', 33994)
GL_TEXTURE11 = Enum('GL_TEXTURE11', 33995)
GL_TEXTURE12 = Enum('GL_TEXTURE12', 33996)
GL_TEXTURE13 = Enum('GL_TEXTURE13', 33997)
GL_TEXTURE14 = Enum('GL_TEXTURE14', 33998)
GL_TEXTURE15 = Enum('GL_TEXTURE15', 33999)
GL_TEXTURE16 = Enum('GL_TEXTURE16', 34000)
GL_TEXTURE17 = Enum('GL_TEXTURE17', 34001)
GL_TEXTURE18 = Enum('GL_TEXTURE18', 34002)
GL_TEXTURE19 = Enum('GL_TEXTURE19', 34003)
GL_TEXTURE2 = Enum('GL_TEXTURE2', 33986)
GL_TEXTURE20 = Enum('GL_TEXTURE20', 34004)
GL_TEXTURE21 = Enum('GL_TEXTURE21', 34005)
GL_TEXTURE22 = Enum('GL_TEXTURE22', 34006)
GL_TEXTURE23 = Enum('GL_TEXTURE23', 34007)
GL_TEXTURE24 = Enum('GL_TEXTURE24', 34008)
GL_TEXTURE25 = Enum('GL_TEXTURE25', 34009)
GL_TEXTURE26 = Enum('GL_TEXTURE26', 34010)
GL_TEXTURE27 = Enum('GL_TEXTURE27', 34011)
GL_TEXTURE28 = Enum('GL_TEXTURE28', 34012)
GL_TEXTURE29 = Enum('GL_TEXTURE29', 34013)
GL_TEXTURE3 = Enum('GL_TEXTURE3', 33987)
GL_TEXTURE30 = Enum('GL_TEXTURE30', 34014)
GL_TEXTURE31 = Enum('GL_TEXTURE31', 34015)
GL_TEXTURE4 = Enum('GL_TEXTURE4', 33988)
GL_TEXTURE5 = Enum('GL_TEXTURE5', 33989)
GL_TEXTURE6 = Enum('GL_TEXTURE6', 33990)
GL_TEXTURE7 = Enum('GL_TEXTURE7', 33991)
GL_TEXTURE8 = Enum('GL_TEXTURE8', 33992)
GL_TEXTURE9 = Enum('GL_TEXTURE9', 33993)
GL_TEXTURE_2D = Enum('GL_TEXTURE_2D', 3553)
GL_TEXTURE_BINDING_2D = Enum('GL_TEXTURE_BINDING_2D', 32873)
GL_TEXTURE_BINDING_CUBE_MAP = Enum('GL_TEXTURE_BINDING_CUBE_MAP', 34068)
GL_TEXTURE_CUBE_MAP = Enum('GL_TEXTURE_CUBE_MAP', 34067)
GL_TEXTURE_CUBE_MAP_NEGATIVE_X = Enum('GL_TEXTURE_CUBE_MAP_NEGATIVE_X', 34070)
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = Enum('GL_TEXTURE_CUBE_MAP_NEGATIVE_Y', 34072)
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = Enum('GL_TEXTURE_CUBE_MAP_NEGATIVE_Z', 34074)
GL_TEXTURE_CUBE_MAP_POSITIVE_X = Enum('GL_TEXTURE_CUBE_MAP_POSITIVE_X', 34069)
GL_TEXTURE_CUBE_MAP_POSITIVE_Y = Enum('GL_TEXTURE_CUBE_MAP_POSITIVE_Y', 34071)
GL_TEXTURE_CUBE_MAP_POSITIVE_Z = Enum('GL_TEXTURE_CUBE_MAP_POSITIVE_Z', 34073)
GL_TEXTURE_MAG_FILTER = Enum('GL_TEXTURE_MAG_FILTER', 10240)
GL_TEXTURE_MIN_FILTER = Enum('GL_TEXTURE_MIN_FILTER', 10241)
GL_TEXTURE_WRAP_S = Enum('GL_TEXTURE_WRAP_S', 10242)
GL_TEXTURE_WRAP_T = Enum('GL_TEXTURE_WRAP_T', 10243)
GL_TRIANGLES = Enum('GL_TRIANGLES', 4)
GL_TRIANGLE_FAN = Enum('GL_TRIANGLE_FAN', 6)
GL_TRIANGLE_STRIP = Enum('GL_TRIANGLE_STRIP', 5)
GL_TRUE = Enum('GL_TRUE', 1)
GL_UNPACK_ALIGNMENT = Enum('GL_UNPACK_ALIGNMENT', 3317)
GL_UNSIGNED_BYTE = Enum('GL_UNSIGNED_BYTE', 5121)
GL_UNSIGNED_INT = Enum('GL_UNSIGNED_INT', 5125)
GL_UNSIGNED_SHORT = Enum('GL_UNSIGNED_SHORT', 5123)
GL_UNSIGNED_SHORT_4_4_4_4 = Enum('GL_UNSIGNED_SHORT_4_4_4_4', 32819)
GL_UNSIGNED_SHORT_5_5_5_1 = Enum('GL_UNSIGNED_SHORT_5_5_5_1', 32820)
GL_UNSIGNED_SHORT_5_6_5 = Enum('GL_UNSIGNED_SHORT_5_6_5', 33635)
GL_VALIDATE_STATUS = Enum('GL_VALIDATE_STATUS', 35715)
GL_VENDOR = Enum('GL_VENDOR', 7936)
GL_VERSION = Enum('GL_VERSION', 7938)
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = Enum('GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING', 34975)
GL_VERTEX_ATTRIB_ARRAY_ENABLED = Enum('GL_VERTEX_ATTRIB_ARRAY_ENABLED', 34338)
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = Enum('GL_VERTEX_ATTRIB_ARRAY_NORMALIZED', 34922)
GL_VERTEX_ATTRIB_ARRAY_POINTER = Enum('GL_VERTEX_ATTRIB_ARRAY_POINTER', 34373)
GL_VERTEX_ATTRIB_ARRAY_SIZE = Enum('GL_VERTEX_ATTRIB_ARRAY_SIZE', 34339)
GL_VERTEX_ATTRIB_ARRAY_STRIDE = Enum('GL_VERTEX_ATTRIB_ARRAY_STRIDE', 34340)
GL_VERTEX_ATTRIB_ARRAY_TYPE = Enum('GL_VERTEX_ATTRIB_ARRAY_TYPE', 34341)
GL_VERTEX_SHADER = Enum('GL_VERTEX_SHADER', 35633)
GL_VIEWPORT = Enum('GL_VIEWPORT', 2978)
GL_ZERO = Enum('GL_ZERO', 0)


ENUM_MAP = {}
for ob in list(globals().values()):
    if repr(ob).startswith('GL_'):
        ENUM_MAP[int(ob)] = ob
del ob
